#!/usr/bin/perl -w

use strict;
use warnings;
use CGI qw(:standard);
use LWP::UserAgent;
use JSON;

sub decodeJSON {
  my ($JSONText) = @_;
  my $hashRef = decode_json($JSONText);
  return $hashRef;
}
sub encodeJSON{
  my($arrayRef) = @_;
  my $JSONText= JSON->new->utf8->encode($arrayRef);
  return $JSONText;
}

my $baseDir = '/var/CommuniGate';
my $nmPath = 'private/notmuch';
my $configName = 'config';
my $sortDefault = 'newest-first';

print "Content-Type: text/plain\n\n";

my $ipAddr = $ENV{'REMOTE_ADDR'};
# accept requests only from localhost
if(!$ipAddr || !$ipAddr =~ m/127\.0\.0\.1/) {
  print "requests from $ipAddr are not allowed";
  exit;
}

my $operation = param('operation');
my $accountName = param('accountName');
my $domainName = param('domainName');
# sessionId is used to store current search state only
my $sessionId = param('sessionId');
unless($operation && $accountName && $domainName && $sessionId) {
  print "bad params: $operation, $accountName, $domainName, $sessionId";
  exit;
}
my $sort = param('sort') || $sortDefault;
my $field = param('field');

my $config;
my $result;
if(-d "$baseDir/Domains/$domainName/$accountName.macnt") {
  $config = "$baseDir/Domains/$domainName/$accountName.macnt/account.web/$nmPath/$configName";
  $result = "$baseDir/Domains/$domainName/$accountName.macnt/account.web/$nmPath/$sessionId";
} else {
  # main domain
  $config = "$baseDir/Accounts/$accountName.macnt/account.web/$nmPath/$configName";
  $result = "$baseDir/Accounts/$accountName.macnt/account.web/$nmPath/$sessionId";
}
unless(-f $config) {
  print "notmuch config for $accountName\@$domainName is missing";
  exit;
}
if(-f $result) {
  print "notmuch is locked for sessionId $sessionId";
  exit;
}

my $cmd = "export NOTMUCH_CONFIG=$config; notmuch ";
if($operation eq 'index') {
  $cmd .= 'new ';
} elsif($operation eq 'search') {
  $cmd .= "search --output=files --format=json --sort=$sort ";
  my $folder = param('folder');
  if($folder) {
    $cmd .= "folder:$folder ";
  }
  my $query = param('query');
  unless($query) {
    print "query param is missing";
    exit;
  }
  if($field) {
    $cmd .= "$field:";
  }
  $cmd .= $query;
}
# redirect output to result file
$cmd .= "> $result";
if(system($cmd)) {
  print "failed to execute cmd: $!";
  print "\n$cmd";
  exit;
}
my $fh;
if(!open($fh, '<', $result)) {
  print "failed to open result file";
  exit;
}
my $str = '';
while(my $row = <$fh>) {
  $str .= $row;
}
unlink $result;
if($operation eq 'index') {
  print $str;
} elsif($operation eq 'search') {
  my $data = decodeJSON($str);
  my @result;
  foreach my $path (@$data) {
    $path =~ s/^.*$accountName\.macnt\///;
    my @p = split(/\//, $path);
    my %m;
    $p[$#p] =~ s/^(\d+)\-.*$/$1/;
    $m{uid} = $p[$#p];
    delete $p[$#p];
    $m{mailbox} = join('/', @p);
    $m{mailbox} =~ s/\.folder//;
    $m{mailbox} =~ s/\.mdir//;
    push(@result, \%m);
  }
  print encodeJSON(\@result);
}
exit;
